/*
 * =====================================================================================
 *
 *       Filename:  save_raw_image.c
 *
 *    Description:  save_raw_image c file
 *
 *        Version:  1.1
 *
 *         Author:  Alexander Aring, a.aring@phytec.de
 *    Modified by:  Dirk Bender, d.bender@phytec.de
 *        Company:  PHYTEC
 *
 *	  History:  1.0 Initial 
 *		    1.1 modified for PHYTEC PD13.1.1 phyFLEX-i.MX6	
 *
 * =====================================================================================
 */

#include "save_raw_image.h"

static char save_image_file_path[PATH_MAX] = DEFAULT_SAVE_FILE;
/* TODO parameter */
char *register_timeout_file_path = NULL;

//static char *modules_to_load[] = {
//   "mx2_camera",
//   NULL
//};

//static char *modules_to_unload[] = {
//   "mx2_camera",
//   "tw9910",
//   "mt9m111",
//   "mt9v022",
//   "mt9m001",
//   NULL
//};

/*-----------------------------------------------------------------------------
 *  What we do with the image.
 *-----------------------------------------------------------------------------*/
static int do_on_capture(const void *image, size_t imagesize)
{
	int err;
	size_t err2;
	FILE *file = NULL;
	
	file = fopen(save_image_file_path, "w+b");
	if (!file) {
		perror("fopen");
		goto out;
	}
	   	
	
	err2 = fwrite(image, imagesize, 1, file);
	if (!err2) {
		fprintf(stderr, "Cannot write image to file");
		goto close;
	}

	err = fclose(file);
	if (err == EOF) {
		perror("fclose");
		goto out;
	}
	
	return 0;
close:
	fclose(file);
out:
	return -1;
}

/*-----------------------------------------------------------------------------
 *  What we do with a register line.
 *-----------------------------------------------------------------------------*/
int write_cam_reg(__u8 address, __u8 reg,
		__u16 val, char mode)
{
	int err;
	__u64 readed = 0;

	err = get_cam_reg(address, reg, &readed);
	if (err < 0)
		return -1;

	switch (mode) {
	case 'a':
		val &= readed;
		break;
	case 'o':
		val |= readed;
		break;
	case 'x':
		val ^= readed;
		break;
	default:
		/* Do nothing */
		break;
	}

	err = set_cam_reg(address, reg, val);
	if (err < 0)
		return -1;

	return 0;
}

static int parse_resolution(char *str, __u32 *width, __u32 *height)
{
	char *fst;
	char *scd;

	fst = str;
	scd = strchr(str, 'x');

	if (!scd)
		return -1;

	*scd = '\0';
	scd++;

	*width = strtoul(fst, NULL, 10);
	if (*width == ULONG_MAX)
		return -1;
	*height = strtoul(scd, NULL, 10);
	if (*height == ULONG_MAX)
		return -1;
	
	return 0;
}

/*-----------------------------------------------------------------------------
 *
 *  Main function of save_raw_image.
 *
 *  Parameter for save_raw_image: 
 *  -d Dropouts to save the image  
 *  -f Pixel FORMAT
 *  -r Name of REGISTERSETTINGS File 
 *  -n Name for Image
 *  -t Timeout
 *  -T Name of the registersettings that will be load after a timeout
 *  -D Video Device e.g. /dev/video0
 *  -R Resolution e.g. 1280,1024
 *  
 *-----------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
	int opt, err, resflag = 0;
	unsigned int count = 1, timeout = 0;
	char *register_file_path = NULL;
	char video_dev[PATH_MAX] = DEFAULT_VIDEO_DEVICE;
	__u32 pixel_format = V4L2_PIX_FMT_GREY;
	__u32 width = 0, height = 0;

	/* Getting parameters */
	while ((opt = getopt(argc, argv, "n:r:f:d:D:t:R:T:")) != -1) {
		switch (opt) {
		case 'n':
			strncpy(save_image_file_path, optarg, PATH_MAX);
			break;
		case 'r':
			register_file_path = optarg;
			break;
		case 'T':
			register_timeout_file_path = optarg;
			break;
		case 'f':
			if (!strcmp("bggr", optarg))
				pixel_format = V4L2_PIX_FMT_SBGGR8;
			if (!strcmp("grbg", optarg))
                                pixel_format = V4L2_PIX_FMT_SGRBG8;
			if (!strcmp("gbrg", optarg))
                                pixel_format = V4L2_PIX_FMT_SGBRG8;
			if (!strcmp("rggb", optarg))
                                pixel_format = V4L2_PIX_FMT_SRGGB8;
			if (!strcmp("yuyv", optarg))
				pixel_format = V4L2_PIX_FMT_YUYV;
			if (!strcmp("yvyu", optarg))
				pixel_format = V4L2_PIX_FMT_YVYU;
			if (!strcmp("uyvy", optarg))
				pixel_format = V4L2_PIX_FMT_UYVY;
			if (!strcmp("vyuy", optarg))
				pixel_format = V4L2_PIX_FMT_VYUY;
			if (!strcmp("rgb", optarg))
				pixel_format = V4L2_PIX_FMT_RGB565;
			break;
		case 'D':
			strncpy(video_dev, optarg, PATH_MAX);
			break;
		case 'R':
			resflag = 1;
			err = parse_resolution(optarg, &width, &height);
			if (err < 0) {
				fprintf(stderr, "Usage: %s "
						SAVE_RAW_IMAGE_USAGE, argv[0]);
				goto out;
			}
			break;
		case 't':
			timeout = strtoul(optarg, NULL, 0);
			if (timeout == ULONG_MAX) {
				perror("strtoul");
				goto out;
			}
			break;
		case 'd':
			count = strtoul(optarg, NULL, 0);
			if (count == ULONG_MAX) {
				perror("strtoul");
				goto out;
			}
			break;
		default: /* '?' */
			fprintf(stderr, "Usage: %s "
					SAVE_RAW_IMAGE_USAGE, argv[0]);
			goto out;
		}
	}
//	err = fmutils_modprobe(modules_to_load);
//	if (err < 0)
//		goto out;

	err = open_video_device(video_dev);
	if (err < 0)
		goto err_rmmod;

	err = init_video_device(video_dev);
	if (err < 0)
		goto err_close;

	if (!resflag) {
		err = get_max_framesize(&width, &height, pixel_format);
		if (err < 0) {
			fprintf(stderr, "Format not supported!\n");
			goto err_close;
		}
	}

	if (register_file_path) {
		err = do_reg_file_parse(register_file_path, write_cam_reg);
		if (err <0) {
			fprintf(stderr, "Failed to parse register file!\n");
			goto err_close;
		}
	}

	err = capture_image(timeout, count, width, height,
			pixel_format, do_on_capture);
	if (err < 0) {
		fprintf(stderr, "Capture failed!\n");
		goto err_close;
	}

	err = close_video_device();
	if (err < 0)
		goto err_rmmod;

//	err = fmutils_rmmod(modules_to_unload);
//	if (err < 0)
//		goto out;

	return EXIT_SUCCESS;
err_close:
	close_video_device();
err_rmmod:
//	fmutils_rmmod(modules_to_unload);
out:
	return EXIT_FAILURE;
}

