/*
 * Copyright (c) 2014 PHYTEC Messtechnik GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <opencv/cv.h>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>
#include <string.h>
#include <unistd.h>

using namespace cv;

int main(int argc, char* argv[]) {

	char *camtype = NULL;
	int scale = 0;
	int index;
	int c;
	double frame_width;
	double frame_height;
	double x_div;
	double y_div;

	opterr = 0;
	while ((c = getopt (argc, argv, "c:s")) != -1)
		switch (c)
		{
		case 'c':
			camtype = optarg;
			if (strcmp(camtype,"VM-006")==0){
				frame_width=1280;
				frame_height=1024;
				break;
			}else if ((strcmp(camtype,"VM-009")==0)){
				frame_width=1280;
				frame_height=1024;
				break;
			}else if ((strcmp(camtype,"VM-010")==0)){
				frame_width=752;
				frame_height=480;
				break;
			}else if ((strcmp(camtype,"VM-011")==0)){
				sleep(1);
				fprintf (stderr, "\nVM-011 is not supported at the moment.\n");
				fprintf (stderr, "It is not possible to grab a frame in full resolution.\n");
				fprintf (stderr, "Also the color format is not correct!\n\n");
				sleep(1);
				frame_width=2592;
				frame_height=1944;
				break;
			}else if ((strcmp(camtype,"VM-008")==0)){
				fprintf (stderr, "VM-008 is not supported at the moment.\n");
				return 1;
			}else{
				fprintf (stderr, "-c has an incorret argument.\n");
                                return 1;
			}
                case 's':
                        scale = 1;;
                        break;
		case '?':
			if (optopt == 'c'){
				fprintf (stderr, "-%c requires an argument.\n", optopt);
				return 1;
			}
			else if (isprint (optopt)){
				fprintf (stderr, "Unknown option `-%c'.\n", optopt);
				return 1;
			}
			else{
				fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
				return 1;
			}
		default:
			abort ();
	}

	for (index = optind; index < argc; index++)
		printf ("Non-option argument %s\n", argv[index]);

	if (camtype == NULL){
		fprintf (stderr, "The Name of the Camera Module is missing.\n");
                fprintf (stderr, "Use -c <Name of the Camera Module>\n");
                fprintf (stderr, "Name of the Camera Module: VM-006, VM-009, VM-010 or VM-011.\n");
		fprintf (stderr, "With the parameter -s you can scale the image to 640x480.\n");
		fprintf (stderr, "For example: -c VM-009 -s\n");
		return 1;
	}

	VideoCapture cap(0); // open the default camera

	if ((strcmp(camtype,"VM-011")!=0)){
	        cap.set(CV_CAP_PROP_FRAME_WIDTH, frame_width);
		cap.set(CV_CAP_PROP_FRAME_HEIGHT, frame_height);
	}

	if(!cap.isOpened())  // check if we succeeded
		return 1;
	namedWindow("Detected Edges",1);

	for(;;)
	{
		Mat in;
		Mat edges;
		Mat resized_frame;

		cap >> in; // get a new frame from camera
		cvtColor(in, edges, CV_BGR2GRAY);
		GaussianBlur(edges, edges, Size(7,7), 1.5, 1.5);
		Canny(edges, edges, 0, 30, 3);
		if (scale == 1){
			resizeWindow("Detected Edges", 640, 480);
			x_div = 640/frame_width;
			y_div = 480/frame_height;
			resize(edges,resized_frame,Size(),x_div,y_div,INTER_LINEAR);
			imshow("Detected Edges", resized_frame);
		}
		else{
			imshow("Detected Edges", edges);
		}

		if(waitKey(30) >= 0) break;
	}

	return 0;
}
